
package agtweet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author shauns
 */
public class TweetList implements Iterable<Tweet>{
    private List<Tweet> _tweets;
    private UserList    _userList;
    
    private boolean _tweetsArePublished = false;
    
    class TweetFileException extends Exception 
    {
        public TweetFileException(String message) { super(message); }
    }    
    
    public boolean areTweetsPublished(){ return _tweetsArePublished; }
    
    @Override
    public Iterator<Tweet> iterator() {
        return _tweets.iterator();
    }
    
    public TweetList(UserList userlist) 
    {
        if(userlist==null){ throw new RuntimeException("Userlist must not be null!"); }
        
        _tweets   = new ArrayList<Tweet>();
        _userList = userlist;        
    }
    
    private Tweet ParseTweet( String line, 
                              String errPrefix ) throws TweetFileException
    {
        
        if( line.isEmpty()                           ){ throw new TweetFileException(errPrefix + AGTweet.Strings.TWEETFILE_ERR_EMPTYLINE ); }
        if( !line.contains(AGTweet.Seperators.TWEET) ){ throw new TweetFileException(errPrefix + AGTweet.Strings.TWEETFILE_ERR_MISSINGSEP + " -> " + line); }

        String[] lineSplit  = line.split(AGTweet.Seperators.TWEET);
        
        String name  = lineSplit[0];
        String tweet = lineSplit[1];
               
        try {
             return new Tweet( name, tweet, errPrefix );             
        } catch (Tweet.TweetException te) {                        
            
            /* This is a bit of a redundant rethrow, but it does avoid adding another throwable type to the method signature */
            
            throw new TweetFileException(te.getMessage());
        } 
    }
    
    public void LoadTweets(File tweetFile) throws TweetFileException
    {
        BufferedReader buf = null;        
        try 
        {
            buf = new BufferedReader(new FileReader(tweetFile));

            String line = null;

            int lineNum = 0;

            List<TweetFileException> errors = new ArrayList<TweetFileException>();

            while( (line=buf.readLine()) != null )
            {
                String errPrefix = "Line [" + AGTweet.padRight(Integer.toString(lineNum++), 6) + "]: ";
                
                try
                {
                    Tweet newTweet = ParseTweet(line, errPrefix);
                    
                    if(!_userList.UserExists(newTweet.getUsername()))
                    {
                        throw new TweetFileException( errPrefix + "Non-existant user '" + newTweet.getUsername() + "'" );
                    }
                    
                    _tweets.add(newTweet);
                
                } catch(TweetFileException tfe) {
                    errors.add(tfe);
                }
            }
            
            if(errors.isEmpty())
            {
                Logger.getLogger(AGTweet.LOGNAME).log( Level.INFO, AGTweet.Strings.TWEETFILE_INFO_LOADED_NOERROR );
            }
            else
            {
                Logger.getLogger(AGTweet.LOGNAME).log( Level.INFO, AGTweet.Strings.TWEETFILE_INFO_LOADED_ERROR );
                
                for (TweetFileException userException : errors) 
                {
                    Logger.getLogger(AGTweet.LOGNAME).log( Level.WARNING, userException.getMessage() );
                }
            }            
                
        } catch (IOException ioe) {
            throw new TweetFileException(ioe.getMessage());
        }
        finally {
            try {
                buf.close();
            } catch (IOException ioe) {
                Logger.getLogger(AGTweet.LOGNAME).log(Level.SEVERE, ioe.getMessage(), ioe);
            }
        }        
    }
    
    public void PublishTweets()
    {
        for( Tweet t  : _tweets )
        {
            try {
                
                User u;

                u = _userList.getUser(t.getUsername());
                
                if( AGTweet.Config._selfFollowing )
                {
                    u.RecieveTweet(t);
                }                

                for( String f : u.getFollowers())
                {
                    _userList.getUser(f).RecieveTweet(t);
                }
                    
            } catch (UserList.UserListException ex) {
                Logger.getLogger(TweetList.class.getName()).log(Level.WARNING, ex.getMessage(), ex);
            }                
        }
        
        _tweetsArePublished = true;
    }
       
}
