/*
 *
 */

package agtweet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 *
 * @author shauns
 */
public class UserList implements Iterable<String>{
        
    private SortedMap<String, User> _users;

    private boolean _followersArePopulated = false;
    
    class UserFileException extends Exception 
    {
        public UserFileException(String message) { super(message); }
    }

    class UserListException extends Exception 
    {
        public UserListException(String message) { super(message); }
    }    
    
    public UserList(File userFile) throws UserFileException
    {
        _users = new TreeMap<String, User>();
        
        LoadUsers(userFile);
    }

    public boolean areFollowersPopulated(){ return _followersArePopulated; }
    
    @Override
    public Iterator<String> iterator() {
        return _users.keySet().iterator();
    }    
    
    public User getUser(String username) throws UserListException
    {
        if( !UserExists(username) ){ throw new UserListException(AGTweet.Strings.TWEETLIST_ERR_NOSUCHUSER); }
        
        return _users.get(username);
    }
    
    public boolean UserExists(String username)
    {
        return _users.containsKey(username);
    }
    
    private void ParseUser( String line, 
                            String errPrefix,
                            Map<String,User> userMap ) throws UserFileException
    {
    
        if( line.isEmpty()                            ){ throw new UserFileException(errPrefix + AGTweet.Strings.USERFILE_ERR_USER_EMPTYLINE  ); }
        if( !line.contains(AGTweet.Seperators.FOLLOW) ){ throw new UserFileException(errPrefix + AGTweet.Strings.USERFILE_ERR_USER_MISSINGSEP ); }
        
        String[] lineSplit = line.split(AGTweet.Seperators.FOLLOW);

        String newUserName = lineSplit[0];
        String followers   = lineSplit[1];

        User newUser; 
        
        if(!_users.containsKey(newUserName))
        {
            newUser = new User(newUserName);
            _users.put(newUserName, newUser);
        }
        else
        {
            newUser = _users.get(newUserName);
        }          

        StringTokenizer tok = new StringTokenizer(followers, ",");
        
        while(tok.hasMoreTokens())
        {
            String followerName = tok.nextToken().trim();            
            newUser.AddFollowing(followerName);
            
            /* A bit hacky doing this inline here... But it does work! */
            if( AGTweet.Config._discoverFollowers && 
                !_users.containsKey(followerName) )
            {
                _users.put(followerName, new User(followerName));
            }
        }        
        
    }
    
    private void LoadUsers(File userFile) throws UserFileException
    {
        BufferedReader buf = null;
        try 
        {
            buf = new BufferedReader(new FileReader(userFile));
            
            String line = null;
            
            int lineNum = 0;
            
            List<UserFileException> errors = new ArrayList<UserFileException>();
            
            while( (line=buf.readLine()) != null )
            {
                
                String errPrefix = "Line [" + AGTweet.padRight(Integer.toString(lineNum++), 6) + "]: ";                
                
                try
                {
                    ParseUser(line, errPrefix, _users);
                                    
                } catch(UserFileException ue) {
                    errors.add(ue);
                }
            }
            
            if(errors.isEmpty())
            {
                Logger.getLogger(AGTweet.LOGNAME).log( Level.INFO, AGTweet.Strings.USERFILE_INFO_LOADED_NOERROR );
            }
            else
            {
                Logger.getLogger(AGTweet.LOGNAME).log( Level.WARNING, AGTweet.Strings.USEFILE_INFO_LOADED_ERROR );
                
                for (UserFileException userException : errors) 
                {
                    Logger.getLogger(AGTweet.LOGNAME).log( Level.WARNING, userException.getMessage() );
                }
            }
            
            
        } catch (IOException ioe) {
            throw new UserFileException(ioe.getMessage());
        }
        finally {
            try {
                buf.close();
            } catch (IOException ex) {
                Logger.getLogger(AGTweet.LOGNAME).log(Level.SEVERE, ex.getMessage(), ex);
            }
        }
        
    }
    
    void PopulateFollowers()
    {
        for( String username : _users.keySet() )
        {
            User u = _users.get(username);
            
            for( String f : u.getFollowing() )
            {
                _users.get(f).AddFollower(username);
            }
        }
        
        _followersArePopulated = true;
    }
    
}
